# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand



#' Computes the precision-recall curve (AUC).
#'
#'
#' @param dt A data frame with columns truth (first column) and score (second column).
#' @return A dataframe with precision recall.
#' @export
compute_pr_curve <- function(dt){
  ## --see .SDcols for order
  pred_obj <- prediction(dt$p.adj , dt$isDE)
  perf_obj = performance(pred_obj, measure = "prec", x.measure = "rec")
  data2curve <- data.frame(x.name = perf_obj@x.values[[1]], y.name = perf_obj@y.values[[1]])
  names(data2curve) <- c(unname(perf_obj@x.name), unname(perf_obj@y.name))
  ## -- drop NA
  data2curve <- na.omit(data2curve)
  ## -- add start point
  data2curve <- rbind(c(0,1), data2curve)
  return(data2curve)
}


#' Computes area under the precision-recall curve (AUC).
#'
#' This function calculates the area under the precision-recall curve (AUC).
#'
#' @param dt A data table with columns for recall and precision.
#' @return A numeric value representing the AUC.
#' @export
compute_pr_auc <- function(dt) PRAUC(y_pred = -log10(dt$p.adj) , y_true = dt$isDE)






#' Gets precision-recall objects for a given parameter.
#'
#' This function takes a data table of evaluation parameters and returns precision-recall curves
#' for each term and an aggregate precision-recall curve.
#'
#' @param evaldata_params Data table containing evaluation parameters.
#' @param col_param Column name specifying the parameter for grouping.
#' @param col_truth Column name for binary ground truth values.
#' @param col_score Column name for predicted scores.
#' @return A list containing precision-recall curves and AUCs for each group and an aggregate precision-recall curve and AUC.
#' @importFrom data.table setDT
#' @export
get_pr_object <- function(evaldata_params, col_param = "description", col_truth = "isDE", col_score = "p.adj"  ) {

  ## -- data.table conversion
  dt_evaldata_params <- data.table::setDT(evaldata_params)

  ## -- by params
  pr_curve_params <- dt_evaldata_params[, compute_pr_curve(.SD), by=c("from", col_param), .SDcols=c(col_truth, col_score)]
  pr_auc_params <- dt_evaldata_params[, compute_pr_auc(.SD), by=c("from", col_param), .SDcols=c(col_score, col_truth)]
  names(pr_auc_params)[ names(pr_auc_params) == "V1" ] <- "pr_AUC"

  ## -- aggregate
  pr_curve_agg <- dt_evaldata_params[, compute_pr_curve(.SD), by = "from", .SDcols=c(col_truth, col_score)]
  pr_auc_agg <- dt_evaldata_params[, compute_pr_auc(.SD), by = "from", .SDcols=c(col_score, col_truth)]
  names(pr_auc_agg)[ names(pr_auc_agg) == "V1" ] <- "pr_AUC"

  return(list(byparams = list(pr_curve = as.data.frame(pr_curve_params),
                              pr_auc = as.data.frame(pr_auc_params)),
              aggregate = list(pr_curve = as.data.frame(pr_curve_agg),
                               pr_auc = as.data.frame(pr_auc_agg)))
  )

}



#' Builds a ggplot precision-recall curve.
#'
#' This function takes data frames for precision-recall curve and AUC and builds a ggplot precision-recall curve.
#'
#' @param data_curve Data frame with precision-recall curve.
#' @param data_auc Data frame with AUC.
#' @param palette_color list of colors used.
#' @param ... Additional arguments to be passed to \code{ggplot2::geom_path}.
#' @return A ggplot object representing the precision-recall curve.
#' @importFrom ggplot2 ggplot geom_path geom_text theme_bw xlim ylim scale_color_manual aes sym
#' @export
build_gg_pr_curve <- function(data_curve, data_auc, palette_color = c("#500472", "#79cbb8"), ...){
  
  #print(list(...))
  #print(ggplot2::sym(list(...)))
  #args <- lapply(list(...), function(x) if(!is.null(x)) ggplot2::sym(x) )
  
  
  data_auc <- get_label_y_position(data_auc)
  
  
  ggplot2::ggplot(data_curve) +
    ggplot2::geom_path(ggplot2::aes(x = recall, y = precision , ... ), linewidth = 1) +
    ggplot2::geom_text(data_auc,
                       mapping = ggplot2::aes(x = 0.75, y = pos_y,
                                              label = paste("AUC :", round(pr_AUC, 2) , sep = ""), col = from )
    ) +
    ggplot2::theme_bw() +
    ggplot2::xlim( 0, 1 ) +
    ggplot2::ylim( 0, 1 ) +
    ggplot2::scale_color_manual(values = palette_color)

}



#' Gets precision-recall curves and AUC for both aggregated and individual parameters.
#'
#' This function takes a precision-recall object and returns precision-recall curves and AUCs for both aggregated and individual parameters.
#'
#' @param pr_obj precision-recall object.
#' @param ... Additional arguments to be passed to \code{ggplot2::geom_path}.
#' @return precision-recall curves and AUCs for both aggregated and individual parameters.
#' @importFrom ggplot2 facet_wrap coord_fixed
#' @export
get_pr_curve <- function(pr_obj, ...){

  ## -- aggreg
  data_curve <- pr_obj$aggregate$pr_curve
  data_auc <- pr_obj$aggregate$pr_auc
  pr_obj$aggregate$pr_curve <- build_gg_pr_curve(data_curve, data_auc,  col = from , ... )

  ## -- indiv
  data_curve <- pr_obj$byparams$pr_curve
  data_auc <- pr_obj$byparams$pr_auc
  pr_obj$byparams$pr_curve <- build_gg_pr_curve(data_curve, data_auc,  col = from , ... ) +
                                ggplot2::facet_wrap(~description) +
                                ggplot2::coord_fixed()

  return(pr_obj)
}




